
{app} = (require "./App.coffee")
utils =  (require "./Utils.coffee")

{TableComponent} = require "./TableComponent.coffee"

CURSOR = {}

window.addEventListener "mousemove", (e) ->
	CURSOR =
		x: e.pageX
		y: e.pageY


Sizes = {}
Sizes.base = 10
Sizes.rowHeight = 28
Sizes.indent = 10
Sizes.indentWidth = 1
Sizes.indentHeight = Sizes.rowHeight


# We use a custom convertPoint to deal with old-style scroll positions

Utils.convertPoint = (input, layerA, layerB, context=false) ->

	# Convert a point between two layer coordinate systems

	point = _.defaults(input, {x:0, y:0})

	superLayersA = layerA?.superLayers(context) or []
	superLayersB = layerB?.superLayers(context) or []

	superLayersB.push(layerB) if layerB

	for layer in superLayersA
		point.x += layer.x - layer.scrollFrame.x
		point.y += layer.y - layer.scrollFrame.y

	for layer in superLayersB
		point.x -= layer.x + layer.scrollFrame.x
		point.y -= layer.y + layer.scrollFrame.y

	return point

scroll = new Layer
	width: 400
	height: window.innerHeight
	name: "scroll"

window.addEventListener "resize", =>
	scroll.height = window.innerHeight

window.document.body.addEventListener "mouseleave", ->
	clearTimeout(table._hoverTimer) if table._hoverTimer
	app.bridge.send("layersRollOut", {})

# Yep. Polling indeed. But there is no nice way to get reliable
# events when the mouse enter/leaves the window, unfortunately.

Utils.interval 1, ->

	# We need to inset this, because the reported location is on the edges
	# if we left the document.
	testFrame = Utils.frameInset(scroll.frame, 
		{top:1, left:1, right:1, bottom:1})

	if not Utils.pointInFrame(CURSOR, testFrame)
		app.bridge.send("layersRollOut", {})
		table._unhighlightHover()

scroll.scroll = true
scroll.scrollHorizontal = false
scroll.backgroundColor = app.config.colors.background
scroll.style.borderLeft = "1px solid #{app.config.colors.border}"
scroll.style.borderLeft = "1px solid #000000"

table = new TableComponent
	width: scroll.width
	height: window.innerHeight
	superLayer: scroll
	name: "table"
table.backgroundColor = ""


table.data = []

textColor = (data) ->
	return app.config.colors.text if data.visibleResult
	return app.config.colors.textInvisible

table.totalRows = -> @data.length
table.heightForRow = (index) -> Sizes.rowHeight
table.marginForRow = (index) -> 0
table.createRow = ->
	row = new Layer
		height: table.heightForRow()
	row.style =
		borderBottom: "1px solid #{app.config.colors.border}"
	
	# Indentation layer (for the hierarchy)
	row.indentLayer = new Layer
		x: Sizes.indent
		width: row.width
		height: Sizes.indentHeight
		superLayer: row
	row.indentLayer.style =
		background: "repeating-linear-gradient(to right,
			#{app.config.colors.indent},
			#{app.config.colors.indent} #{Sizes.indentWidth}px,
			#{app.config.colors.background} #{Sizes.indentWidth}px,
			#{app.config.colors.background} #{Sizes.indent}px
		)"
	row.indentLayer.centerY()

	# Text layer (containing the name)
	row.textLayer = new Layer
		width: row.width
		height: table.heightForRow()
		superLayer: row
		backgroundColor: app.config.colors.background
	row.textLayer.style =
		font: "11px HelveticaNeue"
		color: app.config.colors.text
		lineHeight: "#{table.heightForRow()}px"
		letterSpacing: ".4px"
		# textShadow: "0px 1px 1px rgba(0,0,0,0.50)"
		# "-webkit-font-smoothing": "antialiased"
		"-webkit-user-select": "none"

	# Events
	row.on Events.MouseOver, @onMouseOver
	row.on Events.MouseOut, @onMouseOut
	row.on "mousedown", @onClick
	
	row

table.updateRow = (index, row) ->
	
	row.data = @data[index]
	row.textLayer.html = "#{row.data.name}"

	indent = (row.data.level + 1) * Sizes.indent
	row.textLayer.x = indent
	row.textLayer.width = row.width - indent

	if @isRowSelected("hover", row)
		row.textLayer.style.color = app.config.colors.textHighlight
	else
		row.textLayer.style.color = textColor(row.data)

	if @isRowSelected("selection", row)
		row.backgroundColor = row.textLayer.backgroundColor = app.config.colors.tableSelectionBackground
	else
		row.backgroundColor = row.textLayer.backgroundColor = app.config.colors.background

table.onMouseOver = (event, row) => 
	table.setSelectedRows("hover", [row])
	clearTimeout(table._hoverTimer) if table._hoverTimer
	clearTimeout(table._rolloutTimer) if table._rolloutTimer

	table._hoverTimer = Utils.delay 0.2, ->
		return if CURSOR.y <= 0
		return unless Utils.pointInFrame(CURSOR, Utils.convertPoint(row.frame, row, null, context=false))
		table._highlightedLayerId = row.data.id
		app.bridge.send("ui:highlight", {id:row.data.id, data: row.data.__framerInstanceInfo})

table.onMouseOut = (event, row) ->
	table._unhighlightHover()
	clearTimeout(table._rolloutTimer) if table._rolloutTimer
	table._rolloutTimer = Utils.delay 0.1, ->
		app.bridge.send("layersRollOut", {})

table.onClick = (event, row) ->
	# table.setSelectedRows("selection", [row])
	app.bridge.send("layerClick", {id:row.data.id, data: row.data.__framerInstanceInfo})

table._unhighlightHover = ->

	for row in table.getSelectedRows("hover")
		app.bridge.send("ui:unhighlight", {
			id:row.data.id,
			data: row.data.__framerInstanceInfo
		})

	table.setSelectedRows("hover", [])
	table._highlightedLayerId = null
	clearTimeout(table._hoverTimer) if table._hoverTimer


app.views.layerTable = table
	
app.bridge.on "ui:layers", (data) ->

	# We need to check if the current highlighted layer is still existing.
	# If not we need to reset the highlighter.
	if table._highlightedLayerId and table._highlightedLayerId not in _.pluck(data, "id")
		app.bridge.send("ui:unhighlight", {})

	# Sort the layers by hierarchy and render the table view again
	app.views.layerTable.data = utils.sortLayersByHierachy(data)	
	app.views.layerTable.render()

app.bridge.on "panelDidExpand", (expand) ->
	table.setSelectedRows("selection", []) if expand is false

if Utils.isChrome() or Utils.isSafari()

	data = []

	for i in [1..30]
		data.push
			id:i
			name:"Layer #{i}"
			visibleResult:true
			level:_.random(1,3)
			states:
				one: {x:100, y:60}
				two: {opacity:0.5}

	app.views.layerTable.data = data
	app.views.layerTable.render()
