EventEmitter = require "eventemitter3"
{bridge} = require "./Bridge"

BUILDS = 0

parseUrl = (url) ->
	# https://gist.github.com/jlong/2428561
	parser = document.createElement("a")
	parser.href = url
	return parser

class Runtime extends EventEmitter

	constructor: ->
		@setup()

	setup: ->

		# Listen for device changes and pass them on to the viewer so we 
		# can update the menus in the toolbar
		if Framer?.Device
			properties = [ "deviceScale", "contentScale", "deviceType", 
				"keyboard", "orientation", "fullScreen"]
			properties.map (propertyName) ->
				Framer.Device.on "change:#{propertyName}", ->
					bridge.send("device:change")

			Framer.Device.on "change:phoneScale", (phoneScale) ->
				bridge.send("change:phoneScale", {phoneScale: phoneScale})
			Framer.Device._calculatePhoneScale()

		# Tell Framer Studio we are ready
		bridge.send("runtime.init")

		# Set up the error handler
		@_errorHandlerSetup()

	reset: ->
		Utils.reset()

	_errorHandler: (runtimeError) =>

		# This function catches runtime errors, so after the coffeescript was successfully
		# compiled without syntax errors. We have to detect if they come from the compiled
		# coffeescript or from any other library (most likely framer.js). The way to detect
		# it seems to be the .filename property, which is equal to the current location if
		# the error comes from the compiled coffeescript.

		errorFromCompiledCoffeeScript = (runtimeError.filename == window.location.href)

		error = 
			message: runtimeError.message
			lineNumber: runtimeError.lineno
			colNumber: runtimeError.colno
			fileName: runtimeError.filename

		if errorFromCompiledCoffeeScript
			error.type = "coffeescript"
			error.clientCodeId = window.__framerClientCodeId

		else
			fileName = _.last(parseUrl(runtimeError.filename).pathname.split("/"))
			error.type = "javascript"
			error.message = "[#{fileName}] #{runtimeError.message}"

		bridge.send("runtime.error", error)

	_errorHandlerSetup: ->
		window.addEventListener("error", @_errorHandler)

	_errorHandlerRemove: ->
		window.removeEventListener("error", @_errorHandler)

exports.runtime = new Runtime()
