

function runningApp(appIdentifier) {
  return [[NSRunningApplication runningApplicationsWithBundleIdentifier:appIdentifier] firstObject];
}

function getSketchAppBundlePath(appIdentifier) {
  return [[NSWorkspace sharedWorkspace] absolutePathForAppBundleWithIdentifier:appIdentifier];
}

function getSketchAppBundleInfo(appIdentifier) {
  
  var path = getSketchAppBundlePath(appIdentifier);

  if (!path) {
    return;
  }

  return [[NSBundle bundleWithPath:path] infoDictionary];
}

function getSketchApp(appIdentifier) {

  if (appIdentifier == "com.bohemiancoding.sketch3.beta") {
    sketchApp = [COScript app:"Sketch Beta"];
  } else if (appIdentifier == "com.bohemiancoding.sketch3.xcode") {
    sketchApp = [COScript app:"Sketch Xcode Debug"];
  } else {
    sketchApp = [COScript app:"Sketch"];
  }

  return sketchApp;
}

  
var args = [[NSProcessInfo processInfo] arguments];
var appIdentifier;
var scale = 1;

if (args.length < 3) {
  appIdentifier = "com.bohemiancoding.sketch3";
} else {
  appIdentifier = args[2];
}

if (args.length == 4) {
  scale = args[3];
}


log("framersketch: running with " + appIdentifier);

var sketchAppBundleInfo = getSketchAppBundleInfo(appIdentifier);

if (!sketchAppBundleInfo) {
  log("framersketch: Can't get bundle info for, is Sketch installed?")
}

var sketchAppPath = getSketchAppBundlePath(appIdentifier)
var sketchAppVersion = sketchAppBundleInfo["CFBundleVersion"]
var sketchAppVersionString = sketchAppBundleInfo["CFBundleShortVersionString"]

log("framersketch: version:" + sketchAppVersionString + " / " + sketchAppVersion + " path:" + sketchAppPath);
log("framersketch: scale:" + scale);

var sketchApp = getSketchApp(appIdentifier);
var scriptData = [
  "var path;",
  "if ([doc respondsToSelector:\"exportFramerWithOptions:\"]) {",
  "path = [doc exportFramerWithOptions:[NSDictionary dictionaryWithObject:" + scale + " forKey:'scale']];", 
  "} else {",
  "log('WARNING: Cannot use scale " + scale + ", Sketch version too old, falling back to 1');",
  "path = [doc exportFramer];",
  "}",
  "log('TEMP_DIR:'+path);",
  "log('FILE_DIR:'+[[doc fileURL] path]);",
].join("\n");

// The script only works if you run this "naked" outside a function
[[sketchApp delegate] runPluginScript:scriptData]