#!/usr/bin/env python

import os
import sys
import shutil
import subprocess
import json
import re




if len(sys.argv) < 4:
	sys.exit("Usage run.py <path> <identifier> <scale>")

outPath = os.path.normpath(sys.argv[1])
appIdentifier = os.path.normpath(sys.argv[2])
scale = os.path.normpath(sys.argv[3])

path = os.path.dirname(os.path.abspath(__file__))

print "Writing project to '%s'" % outPath

os.chdir(path)

def run(command):
	return subprocess.check_output(command, shell=True)

def getFilePathFromOutput(output):
	prefix = "FILE_DIR:"
	for line in output.splitlines():
		if line.startswith(prefix):
			return line.lstrip(prefix).strip()

def getTempPathFromOutput(output):
	prefix = "TEMP_DIR:"
	for line in output.splitlines():
		if line.startswith(prefix):
			return line.lstrip(prefix).strip()

def applescript(input):
	run("osascript<<END\n%s\nEND" % input)

def sendToTrash(path):
	applescript('tell application "Finder" to delete POSIX file "%s"' % path)

def sanitizeLayerName(str):

	# Sanitize the layer names so mylayer.jpg gets converted to mylayer
	for suffix in [".jpg", ".pdf"]:
		str = re.sub(suffix, "", str, flags=re.I)

	return re.sub("[^0-9a-zA-Z]", "", str)

def processOutput(documentPath, path):

	filePath = os.path.join(path, "layers.json")

	name = os.path.basename(os.path.normpath(documentPath)).replace(".sketch", "")
	data = json.load(open(filePath, "r"))

	def traverse(layers):
		for layer in layers:
			layer["name"] = sanitizeLayerName(layer["name"])
			if layer.has_key("children"):
				layer["children"] = traverse(layer["children"])

		return layers

	data = traverse(data)
	data = json.dumps(data, sort_keys=True, indent=4)

	open(filePath, "w").write(data)

	TPL = u"""
window.__imported__ = window.__imported__ || {};
window.__imported__["%s/layers.json.js"] = %s
""" % (name, data)

	open(filePath + u".js", "w").write(TPL)


def main():

	print "Processing %s at scale %s" % (path, scale)

	output = run("./coscript run.js %s %s" % (appIdentifier, scale))

	print output

	tempPath = getTempPathFromOutput(output)
	filePath = getFilePathFromOutput(output)

	if not tempPath:
		raise Exception("Could not get temporary path from Sketch. Please restart Sketch and Framer and try again.")

	if not filePath:
		raise Exception("Could not get file path from Sketch. Please restart Sketch and Framer and try again.")

	# run("open '%s'" % tempPath)

	if os.path.exists(outPath):
		# sendToTrash(outPath)
		shutil.rmtree(outPath)

	shutil.copytree(tempPath, outPath)

try:
	main()
except Exception as e:
	import traceback
	traceback.print_exc()
